@@ -5,6 +5,7 @@
 #
 import ast
 from collections import namedtuple
+import copy
 import io
 import itertools
 import json
@@ -13,7 +14,7 @@ import sys
 import textwrap
 from types import SimpleNamespace
 import unittest
-from typing import Optional, Union
+from typing import Optional, Union, NamedTuple
 
 import littletable as lt
 
@@ -89,6 +90,13 @@ else:
 
 DataTuple = namedtuple("DataTuple", "a b c")
 
+
+class DataNamedTuple(NamedTuple):
+    a: int
+    b: int
+    c: int
+
+
 # if rich is not installed, disable table.present() calls
 try:
     import rich
@@ -167,7 +175,7 @@ class TestTableTypes(unittest.TestCase):
 def announce_test(fn):
     def _inner(*args):
         print("\n" + "-" * 50)
-        print(fn.__qualname__)
+        print("{}.{}".format(type(args[0]).__name__, fn.__name__))
         return fn(*args)
 
     return _inner
@@ -190,6 +198,7 @@ def make_test_class(*classes):
 def make_test_classes(cls):
     make_test_class(cls, UsingDataObjects)
     make_test_class(cls, UsingNamedtuples)
+    make_test_class(cls, UsingDataNamedtuples)
     make_test_class(cls, UsingSlottedObjects)
     make_test_class(cls, UsingSimpleNamespace)
     if dataclasses is not None:
@@ -220,6 +229,10 @@ class UsingNamedtuples(AbstractContentTypeFactory):
     data_object_type = DataTuple
 
 
+class UsingDataNamedtuples(AbstractContentTypeFactory):
+    data_object_type = DataNamedTuple
+
+
 class UsingSlottedObjects(AbstractContentTypeFactory):
     data_object_type = Slotted
 
@@ -615,6 +628,8 @@ class TableListTests:
     def test_index_find(self):
         self._test_init()
         self.assertEqual(13, self.t1.index(self.test_rec), "failed 'index; test (exists)")
+        if isinstance(self.test_rec, SimpleNamespace):
+            self.assertEqual(13, self.t1.index(vars(self.test_rec)), "failed 'index; test (exists)")
 
         no_such_rec = self.make_data_object(self.test_size+1, self.test_size+1, self.test_size+1)
         with self.assertRaises(ValueError, msg="failed 'index' test (not exists)"):
@@ -633,6 +648,10 @@ class TableListTests:
         self.t1.remove(no_such_rec)
         self.assertEqual(prev_len-1, len(self.t1), "failed removing non-existent record from table (len)")
 
+        if isinstance(self.test_rec, SimpleNamespace):
+            self.t1.remove(vars(self.test_rec))
+            self.assertEqual(prev_len-1, len(self.t1), "failed to remove record as dict from table (len)")
+
     def test_index_access(self):
         self._test_init()
         self.assertEqual(self.test_rec, self.t1[13], "failed index access test")
@@ -640,6 +659,31 @@ class TableListTests:
     def test_count(self):
         self._test_init()
         self.assertTrue(self.t1.count(self.test_rec) == 1, "failed count test")
+        if isinstance(self.test_rec, SimpleNamespace):
+            self.assertTrue(self.t1.count(vars(self.test_rec)) == 1, "failed count test")
+
+    def test_del(self):
+        self._test_init()
+        before_del_len = len(self.t1)
+        del self.t1[13]
+        self.assertFalse(self.test_rec in self.t1, "failed del test")
+        self.assertEqual(before_del_len - 1, len(self.t1))
+
+    def test_pop(self):
+        self._test_init()
+        before_pop_len = len(self.t1)
+        obj = self.t1.pop(13)
+        self.assertFalse(obj in self.t1)
+        self.assertEqual(self.test_rec, obj)
+        self.assertEqual(before_pop_len - 1, len(self.t1))
+
+    def test_pop_last(self):
+        self._test_init()
+        before_pop_len = len(self.t1)
+        expected_pop = copy.copy(self.t1[-1])
+        obj = self.t1.pop()
+        self.assertEqual(expected_pop, obj)
+        self.assertEqual(before_pop_len - 1, len(self.t1))
 
     def test_reversed(self):
         self._test_init()
@@ -1544,6 +1588,7 @@ class TableImportExportTests:
         imports = [
             ("abc.csv", lt.ImportSourceType.file),
             ("abc.tsv", lt.ImportSourceType.file),
+            ("abc.xlsx", lt.ImportSourceType.file),
             ("abc.csv.zip", lt.ImportSourceType.zip),
             ("abc.csv.gz", lt.ImportSourceType.gzip),
             ("abc.csv.xz", lt.ImportSourceType.lzma),
@@ -1554,8 +1599,10 @@ class TableImportExportTests:
                 import_name = "test/" + fname
             else:
                 import_name = fname
-            if not import_name.endswith(".tsv"):
+            if import_name.endswith(".csv"):
                 tbl = lt.Table().csv_import(import_name)
+            elif import_name.endswith(".xlsx"):
+                tbl = lt.Table().excel_import(import_name)
             else:
                 tbl = lt.Table().tsv_import(import_name)
 
@@ -1568,22 +1615,66 @@ class TableImportExportTests:
             self.assertEqual(expected_type, tbl.import_source_type)
 
     def test_csv_import_from_url(self):
-        import subprocess
+        from http.server import HTTPServer, BaseHTTPRequestHandler
+        from http import HTTPStatus
+        import threading
+        import time
+        import urllib.error
         import urllib.request
 
+        class CSVTestRequestHandler(BaseHTTPRequestHandler):
+            def do_GET(self):
+                self.log_message("received %s %s", self.command, self.path)
+                path = self.path
+                if path == "/EXIT":
+                    self.send_response(HTTPStatus.OK)
+                    self.end_headers()
+                    threading.Thread(target=lambda: self.server.shutdown()).start()
+
+                elif path == "/":
+                    self.send_response(HTTPStatus.OK)
+                    self.end_headers()
+
+                elif path.startswith("/abc.csv"):
+                    send_bytes = b"a,b,c\n1,2,3\n"
+                    self.send_response(HTTPStatus.OK)
+                    self.end_headers()
+                    self.wfile.write(send_bytes)
+
+        def run(server_class=HTTPServer, handler_class=CSVTestRequestHandler):
+            server_address = ('', 8888)
+            httpd = server_class(server_address, handler_class)
+            httpd.serve_forever()
+
+        def run_background_test_server():
+            p = threading.Thread(target=run)
+            p.start()
+
+            for tries_remaining in reversed(range(20)):
+                try:
+                    with urllib.request.urlopen("http://localhost:8888/"):
+                        break
+                except urllib.error.URLError:
+                    if tries_remaining:
+                        time.sleep(0.25)
+
+            return p
+
         web_address = "http://localhost:8888"
-        web_server = subprocess.Popen("python test/csv_import_http_server.py".split())
+        p = run_background_test_server()
+
         url = web_address + "/abc.csv"
-        tbl = lt.Table().csv_import(url)
+        try:
+            tbl = lt.Table().csv_import(url)
+        finally:
+            with urllib.request.urlopen(web_address + "/EXIT"):
+                pass
+            p.join()
+
         tbl.present()
         self.assertEqual(url, tbl.import_source)
         self.assertEqual(lt.ImportSourceType.url, tbl.import_source_type)
 
-        with urllib.request.urlopen(web_address + "/EXIT"):
-            pass
-
-        web_server.wait()
-
     def test_csv_filtered_import(self):
         test_size = 3
         tt = lt.Table().csv_import("test/abc.csv", transforms=dict.fromkeys("abc", int))
@@ -1835,6 +1926,34 @@ class TableImportExportTests:
         self.assertTrue(all(make_dataobject_from_ob(rec1) == rec2 for rec1, rec2 in zip(t1, tt)))
         self.assertEqual(len([d for d in data.splitlines() if d.strip()]), len(tt))
 
+    def test_excel_import(self):
+        file_name = "test/abc.xlsx"
+        excel_table = lt.Table().excel_import(file_name, transforms={'a': int, 'b': int, 'c': int})
+
+        test_size = 3
+        t1 = make_test_table(self.make_data_object, test_size)
+
+        self.assertTrue(all(make_dataobject_from_ob(rec1) == rec2 for rec1, rec2 in zip(t1, excel_table)))
+        self.assertEqual(sum(1 for line in csv_data.splitlines() if line.strip())-1, len(excel_table))
+
+        row_prototype = self.make_data_object(0, 0, 0)
+        csvtable2 = lt.Table().excel_import(
+            file_name, transforms={'a': int, 'b': int, 'c': int}, row_class=type(row_prototype)
+        )[:3]
+
+        print(type(t1[0]).__name__, t1[0])
+        print(type(csvtable2[0]).__name__, csvtable2[0])
+        self.assertEqual(type(t1[0]), type(csvtable2[0]))
+
+    def test_excel_export(self):
+        file_name = "test/abc.xlsx"
+        excel_table = lt.Table().excel_import(file_name, transforms={'a': int, 'b': int, 'c': int}, limit=1)
+        outfile = io.BytesIO()
+        excel_table.excel_export(outfile)
+        exported_table = lt.Table().excel_import(outfile, transforms={'a': int, 'b': int, 'c': int})
+
+        self.assertTrue(all(make_dataobject_from_ob(rec1) == rec2 for rec1, rec2 in zip(exported_table, excel_table)))
+
 
 @make_test_classes
 class TablePivotTests:
@@ -1981,6 +2100,9 @@ class TableSearchTests_DataObjects(TableSearchTests, UsingDataObjects):
 class TableSearchTests_Namedtuples(TableSearchTests, UsingNamedtuples):
     pass
 
+class TableSearchTests_DataNamedtuples(TableSearchTests, UsingDataNamedtuples):
+    pass
+
 class TableSearchTests_Slotted(TableSearchTests, UsingSlottedObjects):
     pass
 
@@ -1991,6 +2113,24 @@ if dataclasses is not None:
     class TableSearchTests_Dataclasses(TableSearchTests, UsingDataclasses):
         pass
 
+if pydantic is not None:
+    class TableSearchTests_PydanticModels(TableSearchTests, UsingPydanticModel):
+        pass
+
+    class TableSearchTests_PydanticImmutableModels(TableSearchTests, UsingPydanticImmutableModel):
+        pass
+
+    class TableSearchTests_PydanticORMModels(TableSearchTests, UsingPydanticORMModel):
+        pass
+
+if attr is not None:
+    class TableSearchTests_AttrClasses(TableSearchTests, UsingAttrClass):
+        pass
+
+if traitlets is not None:
+    class TableSearchTests_TraitletsClasses(TableSearchTests, UsingTraitletsClass):
+        pass
+
 
 class InitialTest(unittest.TestCase):
     from littletable import (
